/* PROGRAMME HORLOGE DCF77
* Auteur : Pascal Chour (www.pascalchour.fr)
* objet:          utilitaires pour le programme d'horloge DCF77
* date:           08/2023
*/
#include <msp430f449.h>
#include "common.h"
#include <stdint.h>
#include "dcf77util.h"
#include "lcd.h"



static uint32_t TimerMilli;                 // Valeur du timer en millisecondes

 /*
 * Active (on = TRUE) ou dsactive (on = FALSE) le Timer de 1ms (A0)
 * L'activation du timer active XT2 comme source de SMCLK qui est utilis pour le timer.
 * Dsactive XT2 si on = FALSE
 */


void Timer1ms(bool_t on) {
    if (on) {
        //Initialisation du Timer A0 et de SMCLK
        if (!(TA0CCTL0 & CCIE)) {               // On initialise que si le timer n'est pas dj initialis
            FLL_CTL1 = 0x04;                    // SMCLK ON, XT2 ON, MCLK = DCO, SMCLK = XT2, ALCK/1
            do                                              // wait in loop until crystal is stable
            {
                IFG1 &= ~OFIFG;
            }
            while (IFG1 & OFIFG);

            IFG1 &= ~OFIFG;                     // clear osc. fault int. flag (?)
            TimerMilli = 0;                     // On dmarre le timer  0
            TA0CCR0 = 1000;                     // 8MHz / 8 = 1MHz. 1ms = 1000 pulse d'horloge
            TA0CTL = TBSSEL_2 + MC_2 + ID_3 ;   // SMCLK, continuous mode, SMCLK/8
            TA0CCTL0 = CCIE;                    // Autorise les IT

        }
    } else {
        TA0CTL = 0;                         // Tout inactif pour le timer A0
        TA0CCTL0 = 0;
        TA0CCR0 = 0;
        TimerMilli = 0;                     // Utile pour les tests
        FLL_CTL1 = 0x60;                    // SMCLK OFF, XT2 OFF, MCLK = DCO, SMCLK = DCO, ALCK/1
        IFG1 &= ~OFIFG;                     // clear osc. fault int. flag (?)

    }
}

/*
La fonction ci-dessous mesure le temps en prenant en compte un ventuel bouclage de la fonction millis ou micro
On lui donne la valeur de dpart de la valeur  mesurer et la valeur courante de millis ou micro. Elle rend la dure coule en millisecondes ou microseconde.
DureeVrai = courante-dernire
IL FAUT QUE DERNIERE SOIT ANTERIEURE A COURANTE
*/
uint32_t DureeVraie(uint32_t derniere, uint32_t courante) {
  if (courante < derniere) {  // on est pass par zro
    return (0xffffffffu - derniere) + courante;
  } else {
    return courante - derniere;
  }
}

/*
La fonction ci dessous donne la valeur du Timer en ms
*/
uint32_t TimerMS() {
    return TimerMilli;
}


/*
Cette fonction fait un dlai d'attente LOGICIEL  de x ms.
Elle fonctionne pour une frquence donne du processeur qui
doit tre fixe  la compilation. Dans ce programme, elle n'est utilise
qu'au dmarrage du systme lorsque les fonctions timers sous IT ne sont pas encore activables
*/

void delayMs(unsigned int delay)
{
    while (delay--)
    {
        __delay_cycles(FREQ_PROC_DIV_1000);
    }
}

/*
 * Vrification du buffer de rception, dcodage et remplissage des champs par les valeurs reues
 */

bool_t VerifieBufRecep(int BufRecep[])
{
    unsigned int i, Parite;

    // Minutes
    Parite = 0;
    for (i = 28; i <= 21; i--) {
       Parite = Parite + BufRecep[i];
    }
    if ((Parite && 1) != 0) return FALSE;


    // Conversion BCD -> binaire
    MNtemp = (BufRecep[21] | (BufRecep[22] << 1) | (BufRecep[23] << 2) | (BufRecep[24] << 3)) + 10*(BufRecep[25] | (BufRecep[26] << 1) | (BufRecep[27] << 2));
    if (MNtemp > 59) return FALSE;

    // Heures
    Parite = 0;
    for (i = 35; i <= 29; i--) {
       Parite = Parite + BufRecep[i];
    }
    if ((Parite && 1) != 0) return FALSE;
    // Conversion BCD -> binaire
    HHtemp = (BufRecep[29] | (BufRecep[30] << 1) | (BufRecep[31] << 2) | (BufRecep[32] << 3)) + 10*(BufRecep[33] | (BufRecep[34] << 1));
    if (HHtemp > 23) return FALSE;
    // Date

    Parite = 0;
    for (i = 58; i <= 36; i--) {
       Parite = Parite + BufRecep[i];
    }
    if ((Parite && 1) != 0) return FALSE;
    // Conversion BCD -> binaire : Jour
    JJtemp = (BufRecep[36] | (BufRecep[37] << 1) | (BufRecep[38] << 2) | (BufRecep[39] << 3)) + 10*(BufRecep[40] | (BufRecep[41] << 1));
    if (JJtemp > 31) return FALSE;

    // Conversion BCD -> binaire : jour de la semaine (juste pour contrle car pas utilis)
    i = (BufRecep[42] | (BufRecep[43] << 1) | (BufRecep[44] << 2));
    if (i > 7) return FALSE;

    // Conversion BCD -> binaire : Mois
    MMtemp = (BufRecep[45] +(BufRecep[46] << 1) | (BufRecep[47] << 2) | (BufRecep[48] << 3)) + 10*BufRecep[49];
    if (MMtemp > 12) return FALSE;

    // Conversion BCD -> binaire : Anne dans le sicle
    AAtemp = (BufRecep[50] | (BufRecep[51] << 1) | (BufRecep[52] << 2) | (BufRecep[53] << 3)) + 10*(BufRecep[54] | (BufRecep[55] << 1) | (BufRecep[56] << 2) | (BufRecep[57] << 3));
    if (AAtemp > 99) return FALSE; else return TRUE;
}


/*
procdure d'interruption (Timer A0)
*/

// Timer A0 interrupt service routine
#pragma vector=TIMERA0_VECTOR
__interrupt void TimerISRA0 (void)
{
    LPM3_EXIT; // On sort du mode basse consommation
    TimerMilli++; // On incrmente le timer des millisecondes
    if (!ITMilliseconde) TCourantMS = TimerMilli; // On initialise TCourantMS pour le traitement des bits (uniquement si la prcdente IT a t traite)
    ITMilliseconde = TRUE; // On indique qu'une IT est  traiter
    TA0CCR0 +=1000; // On incrmente le timer A0 pour gnrer une nouvelle IT dans 1ms
}

/*
 Cette routine est appele sur front montant et descendant du signal du module DCF77.
 On commence par rnitialiser le TimeOut sur absence/prsence signal et on (r)affiche le bargraph pour signaler qu'on reoit
 quelque chose.

 On limine certaines impulsions (celles qui se produisent juste aprs un changement d'tat car a ne cote pas cher).
 On pourrait faire mieux (liminer celles qui se produisent en cours de rception d'un signal (fait sur une autre implmentation))
 mais l'exprience montre que ce n'est pas trs utile.
 L'ide est que si on attend un signal de 1700ms et qu'une front se dclenche  exemple  1000ms, alors, on attend de voir
 s'il s'agit rellement d'un signal ou simplement d'une impulsion. Si c'est une impulsion, on l'ignore et on continue la mesure du signal attendu.

 Dans PREMIER0, on attend un front descendant. Lorsqu'on le trouve, on mesure la dure jusqu'au prochain front montant. Si cette dure est suprieure
  1700ms, alors, on considre qu'on a trouv le signal de synchronisation. Sinon, on recommence dans l'tat PREMIER0.

 Lorsqu'on a trouv le signal de synchronisation, l'automate se met dans un etat d'attente (TERMINE).

 Il peut en sortir si la rception de la trame est incorrecte (voir l'automate de l'interruption du compteur A0.


 */
// Timer P2.0 interrupt service routine
#pragma vector=PORT2_VECTOR
__interrupt void Port_2(void)
{
    LPM3_EXIT; // On sort du mode basse consommation
    if (P2IFG & 0x01) { // L'IT Concerne P2.0 ?
        if (!ITFront) TCourantFront = TimerMilli; // on met  jour le TCourantFront pour l'automate qui gre les fronts uniquement si la prcdente IT a t traite
        ITFront = TRUE; // On signale qu'il y a une IT  traiter
    }
    P2IFG &= 0xFE; // validation de l'interruption sur P2.0
}

/*
 * procdure d'interruption (watchdog = WDT_VECTOR) appele chaque seconde.

*/
#pragma vector=WDT_VECTOR
__interrupt void watchdog_isr( void )
{
    LPM3_EXIT; // On sort du mode basse consommation
    //P1OUT ^= STATUS_LED; // pour test
    ITWatchdog = TRUE; // On signale qu'il y a une IT  traiter
}





/**
 * Allume ou teint la LED
 * Allume : on = TRUE. Eteint : on = FALSE
 */
void setStatusLed (bool_t on)
{
    if (on) {P1OUT &= ~STATUS_LED;} else {P1OUT |= STATUS_LED;} // switch on/off status LED
}


/*
* Gestion des boutons poussoirs par polling
* Pas de gestion de l'antirebon
* Indique quels sont les poussoirs appuys
* En sortie : masque des boutons apppys
*/
unsigned int buttonHandler ()
{
    return (((~P3IN) & BTN_MASK));
}

/*
Indique si une anne est bissextile ou pas
En entre : anne
En sortie : VRAI = anne bissextile, FAUX = anne normale
*/
bool_t Bissextile(int Annee) {
    return ((Annee & 0x03) == 0) && (((Annee % 100 ) != 0) || ((Annee % 400) == 0));
}


/*
Incrmente une date de 1 jour. En entre Jour, Mois, Annee. En sortie, Jour, Mois, Annee mis  jour
Valable jusqu'en 2400 (qui est une anne bissextile !)
*/
void IncrementeDateUnJour(unsigned int *Jour, unsigned int *Mois, unsigned int *Annee) {
    *Jour += 1;
    if (*Jour < 29) return;
    if (*Mois == 2) { // Ici, jour = 29 ou plus. Est-on en fvrier ?
        if (((*Annee & 0x03) == 0) && (((*Annee % 100 ) != 0) || ((*Annee % 400) == 0))) { // Si anne bissextile
            if (*Jour == 29) return; // Alors si le jour est 29, on accepte
        }
        *Jour = 1; // Sinon, on passe en mars
        *Mois = 3;
        return;
    }
    if (*Jour == 30) return;
    // A partir d'ici, jour = 31 ou plus si bug !
    if ((*Jour == 31) && ((*Mois == 1) || (*Mois == 3) || (*Mois == 5) || (*Mois == 7) || (*Mois == 8) || (*Mois == 10) || (*Mois == 12))) return;  // Si mois de 31 jours, alors ok
    *Jour = 1; // On passe au mois suivant
    *Mois +=1;
    if (*Mois > 12) { // Si on est en dcembre, on passe  l'anne suivante
        *Mois = 1;
        *Annee +=1;
    }
}

/*
Gre le bargraph. Doit tre appel priodiquement (typiquement, 1 seconde) par le programme principal
Si la rception est active et qu'on reoit un signal alors, le nombre de barre affich dpend du nombre de parasites
Voir programme pour les valeurs Nombre de parasites vs nombre de barres
Remet  zro le nombre de parasites
Normalement, elle ne doit tre appeles que si la rception est active (pas de contrle)
*/

void GereBargraph(unsigned int *NombreParasites) {
    if (*NombreParasites == 0) {
        LCD_bargraph(3);
    } else {
        if (*NombreParasites <= 3) {
        LCD_bargraph(2);
        } else {
            LCD_bargraph(1);
        }
    }
    *NombreParasites = 0;
}


/*
Cet utilitaire calcule le niveau lu dans le DAC qui mesure la tension
d'alimentation. La valeur en mv se calcule ainsi

  tension = mult = (ADC12MEM0 * 5000l) / 4095l;

  Elle est prvue pour une tension d'alimentation de 3V (pour les barres de la batterie)

!!! Cette procdure fait des attentes logicielles et du polling !!!

Elle ne devrait pas tre appele trop souvent car elle consomme quelques
centaines de A du fait de l'utilisation du DAC.

Elle rend un rsultat utilisable pour afficher la jauge de la batterie sur l'afficheur 
0 : batterie vide (ou quasiment !)
1 : batterie faible
2 : batterie moyenne
3 : batterie charge

Inspir de http://fixituntilitsbroken.blogspot.fr/2011/08/reading-supply-voltage-using-msp430s.html
- If Vcc = 3.3V, ADC output should be (1.65/2.5)*4095 = 2703
- Therefore (halfVcc/2.5)*4095 = ADC reading and (Vcc/2.5)*4095 = 2*ADC
- So Vcc*4096 = 5*ADC and VCC=5*ADC/4095
*/
unsigned int niveau_batterie()
{
 
    unsigned long resultat;
  // REFON  => gnrateur de rfrence actif
  // REF2_5V : utilise le gnrateur 2,5V
  // ADC12ON => dmarre le convertisseur
  // SHT0_15 => chantillonage 1024
  
  ADC12CTL0 = REFON + REF2_5V + ADC12ON + SHT0_15;  // turn on 2.5V ref, set samp time=1024 cycles
  // SHP => source du signal d'chantillonage interne
  ADC12CTL1 = SHP;                                  // Use sampling timer, internal ADC12OSC
  // SREF_1 => V R+ = Ve REF+ and VR- =AV SS
  // INCH_11 => (AVCC  AVSS ) / 2
  ADC12MCTL0 = SREF_1 + INCH_11;                    // Channel A10, Vcc/2
  delayMs(ADC_VREF_DELAY_MS);                       // 17mSec delay required to Vref capacitors
  ADC12CTL0 |= ENC;                                 // Enable conversions
  ADC12CTL0 |= ADC12SC;                             // Start conversions
  while (!(ADC12IFG & 0x01));                       // Conversion done?

  resultat = ADC12MEM0 & 0x0FFF;
  resultat = (resultat*5000l)/(4095l);
  ADC12CTL0 &= ~ENC;        // On dsactive le convertisseur
  ADC12CTL0 &= ~ADC12ON;
  ADC12CTL0 = 0;
  if (resultat >= 3000) return 4;
  if (resultat >= 2900) return 3;
  if (resultat >= 2800) return 2;
  if (resultat >= 2700) return 1;
  return 0;
}



