/*
* File :        PCF8583.h
* objet :       Constantes pour DCF77
* Auteur :      Pascal Chour
* cr le :     08/08/2023
* Copyright :   (C) Pascal Chour (2023)
*/

#ifndef __INCLUDE_GEIGERUTIL_H
#define __INCLUDE_GEIGERUTIL_H

#include <msp430x44x.h>
#include <stdint.h>
#include "globales.h"

#define ADC_VREF_DELAY_MS (17) // dlai pour stabilisation ADC en ms

/*
 * Allume (TRUE) ou taint (FALSE) la lED de la carte Olimex
 */

void setStatusLed (bool_t on);


/*
* Gestion des boutons poussoirs par polling
* Pas de gestion de l'antirebon
* Indique quels sont les poussoirs appuys
* En sortie : masque des boutons apppys
*/
unsigned int buttonHandler ();


/* 
Cette fonction fait un dlai d'attente LOGICIEL  de x ms.
Elle fonctionne pour une frquence donne du processeur qui
doit tre fixe  la compilation
*/
void delayMs(unsigned int delay);

/*
Active (on = TRUE) ou dsactive (on = FALSE) le Timer de 1ms
*/

void Timer1ms(bool_t on);

/*
La fonction ci-dessous mesure le temps en prenant en compte un ventuel bouclage de la fonction millis ou micro
On lui donne la valeur de dpart de la valeur  mesurer et la valeur courante de millis ou micro. Elle rend la dure coule en millisecondes ou microseconde.
DureeVrai = courante-dernire
*/
uint32_t DureeVraie(uint32_t derniere, uint32_t courante);

/*
La fonction ci dessous donne la valeur du Timer en ms
*/
uint32_t TimerMS();

/*
Cet utilitaire calcule le niveau de la tension d'alimentation
du MSP430 et rend un rsultat utilisable pour l'afficheur
la jauge de la batterie sur l'afficheur 
0 : batterie vide (ou quasiment !)
1 : batterie faible
2 : batterie moyenne
3 : batterie charge

Cette procdure ne devrait pas tre appele trop souvent. Par
exemple, une fois toutes les minutes.
*/
unsigned int niveau_batterie();

/*
 * Vrification du buffer de rception du signal. Vrai si vrification OK
 */
bool_t VerifieBufRecep(int BufRecep[]);

/*
Incrmente une date de 1 jour. En entre Jour, Mois, Annee. En sortie, Jour, Mois, Annee mis  jour
*/
void IncrementeDateUnJour(unsigned int *Jour, unsigned int *Mois, unsigned int *Annee);

/*
Indique si une anne est bissextile ou pas
En entre : anne
En sortie : VRAI = anne bissextile, FAUX = anne normale
*/
bool_t Bissextile(int Annee);

/*
Gre le bargraph. Doit tre appel priodiquement (typiquement, 1 seconde) par le programme principal
Si la rception est active et qu'on reoit un signal alors, le nombre de barre affich dpend du nombre de parasites
Voir programme pour les valeurs Nombre de parasites vs nombre de barres
Remet  zro le nombre de parasites
*/

void GereBargraph(unsigned int *NombreParasites);

#endif // __INCLUDE_GEIGERUTIL_H

