/* PROGRAMME HORLOGE DCF77
  * Auteur : Pascal Chour (www.pascalchour.fr)
  * Gestion afficheur GDC0689
  *
 */
#ifndef __INCLUDE_LCD_H
#define __INCLUDE_LCD_H
#include <msp430x44x.h>
#include <stdint.h>
#include "common.h"

#define LCD_PRINTF_BUF_SIZE    (8)

/// pour tests -  virer
void LCD_test();

/// Pointer to access LCD memory
extern char *LCD;

/**
 * Initializes LCD driver.
 *
 * @author Peter Ivanov, Muneem Shahriar
 */
void LCD_init ();

/*
 * Efface l'afficheur
 */
void LCD_clear ();


/*
 * Allume tous les segments de lafficheur
 */
void LCD_all ();


/**
 * Niveau de batterie
 *
 * @Auteur Pascal Chour
 *
 * Input:
 *   @param level Valid values are 0..3:
 *                0: Zero battery,
 *                1: Low battery,
 *                2: Medium battery,
 *                3: Full battery.
 */
void LCD_showBatteryLevel (unsigned int level);

/*
 *  crit les deux points
 *  0 = afficheur haut (COL1), 1 = afficheur bas (COL2), 2 = afficheur bas (COL3)
*/

void LCD_deuxpoints (unsigned int col, bool_t on);


/**
 * Ecriture d'un caractre sur l'afficheur bas
 *
 * Entre:
 *   position 0..5: 0, le digit le plus  droite
 *   letter : A-Z 0-9 . : , / \
 */
void LCD_writeafficheurbas (unsigned int position, char letter);

/*
 * Ecrit 2 digits  une position donne
 * En entre :
 * position = position du caractre le plus  droite
 * Nombre = valeur  crire (0  99). Rsultat imprvisible si Nombre > 99
 *
 */
void LCD_2digitafficheurbas(unsigned int position, int nombre);


/**
 * Ecriture d'un nombre de 4 digits sur l'afficheur haut
 *
 * Entre:
 *   Nombre : nombre de 0  9999
 *   DeuxPoints : vrai, on affiche les deux points. Faux, on les efface
 */
void LCD_4digitafficheurhaut (int nombre, bool_t DeuxPoints);

/*
 * Effacement de l'afficheur du haut y compris les points
 */

void LCD_effaceafficheurhaut();
/**
 * Write a character buffer to LCD.
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param buf Pointer to buffer.
 *   @param length Length of buffer.
 */
void LCD_write (const char *buf, uint8_t length);
/*void LCD_writesp (const char *buf, uint8_t length);*/


/* affichage ou extinction du "cl" */
void LCD_cle(bool_t on);


/* affichage ou extinction du "antenne" */
void LCD_antenne(bool_t on);


/* Gestion du clignotement de l'afficheur */
void LCD_blink(bool_t on);

/* affichage ou extinction d'un point
 * Les positions possible sont 2  6
*/

void LCD_writepoint(unsigned int position, bool_t on);

/*
Affiche tous les segments du Bargraph et l'antenne
level : 0  3 en fonction du niveau
*/
void LCD_bargraph (unsigned int Niveau);

#endif // __INCLUDE_LCD_H
